/***************************************************************************//**
* \file cy_ipc_config.c
* \version 1.20
*
*  Description:
*  This C file is not intended to be part of the IPC driver. It is the code
*  required to configure the device specific IPC channels for semaphores
*  and pipes.
*
********************************************************************************
* Copyright 2016-2018, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "ipc/cy_ipc_drv.h"
#include "ipc/cy_ipc_pipe.h"
#include "ipc/cy_ipc_sema.h"

#include "cy_ipc_config.h"

#if (CY_IPC_CYPIPE_ENABLE)
static void Cy_IPC_SystemPipeIsr(void);

/* Create an array of endpoint structures */
static cy_stc_ipc_pipe_ep_t cy_ipc_pipe_sysEpArray[CY_IPC_MAX_ENDPOINTS];

#define CY_CYPIPE_DEFAULT_CONFIG    \
{\
    /* .ep0ConfigData */ {\
        /* .ipcNotifierNumber    */  CY_IPC_INTR_CYPIPE_EP0,\
        /* .ipcNotifierPriority  */  CY_IPC_INTR_CYPIPE_PRIOR_EP0,\
        /* .ipcNotifierMuxNumber */  CY_IPC_INTR_CYPIPE_MUX_EP0,\
        /* .epAddress            */  CY_IPC_EP_CYPIPE_CM0_ADDR,\
        /* .epConfig             */  CY_IPC_CYPIPE_CONFIG_EP0\
    },\
    /* .ep1ConfigData */ {\
        /* .ipcNotifierNumber    */  CY_IPC_INTR_CYPIPE_EP1,\
        /* .ipcNotifierPriority  */  CY_IPC_INTR_CYPIPE_PRIOR_EP1,\
        /* .ipcNotifierMuxNumber */  0u,\
        /* .epAddress            */  CY_IPC_EP_CYPIPE_CM4_ADDR,\
        /* .epConfig             */  CY_IPC_CYPIPE_CONFIG_EP1\
    },\
    /* .endpointClientsCount     */  CY_IPC_CYPIPE_CLIENT_CNT,\
    /* .endpointsCallbacksArray  */  cy_ipc_pipe_sysCbArray,\
    /* .userPipeIsrHandler       */  &Cy_IPC_SystemPipeIsr\
}
#endif /* CY_IPC_CYPIPE_ENABLE */

#define USRPIPE_CONFIG    \
{\
    /* .ep0ConfigData */ {\
        /* .ipcNotifierNumber    */  CY_IPC_INTR_USRPIPE_CM0,\
        /* .ipcNotifierPriority  */  IPC_INTR_USRPIPE_PRIOR_CM0,\
        /* .ipcNotifierMuxNumber */  IPC_INTR_USRPIPE_MUX_CM0,\
        /* .epAddress            */  CY_IPC_EP_USRPIPE_ADDR_EP0,\
        /* .epConfig             */  IPC_USRPIPE_CONFIG_CM0\
    },\
    /* .ep1ConfigData */ {\
        /* .ipcNotifierNumber    */  CY_IPC_INTR_USRPIPE_CM4,\
        /* .ipcNotifierPriority  */  IPC_INTR_USRPIPE_PRIOR_CM4,\
        /* .ipcNotifierMuxNumber */  0u,\
        /* .epAddress            */  CY_IPC_EP_USRPIPE_ADDR_EP1,\
        /* .epConfig             */  IPC_USRPIPE_CONFIG_CM4\
    },\
    /* .endpointClientsCount     */  CY_IPC_USRPIPE_CLIENT_CNT,\
    /* .endpointsCallbacksArray  */  ipc_pipe_CbArray,\
    /* .userPipeIsrHandler       */  &IPC_UserPipeIsr\
}

/*******************************************************************************
* Function Name: Cy_IPC_SystemSemaInit
****************************************************************************//**
*
* Initializes the system semaphores. The system semaphores are used by Flash.
*
* This function is called in the SystemInit() function. If the default startup
* file is not used, or SystemInit() is not called in your project,
* call the following three functions prior to executing any flash or EmEEPROM
* write or erase operation:
*  -# Cy_IPC_SystemSemaInit()
*  -# Cy_IPC_SystemPipeInit()
*  -# Cy_Flash_Init()
*
*******************************************************************************/
void Cy_IPC_SystemSemaInit(void)
{
/* Create array used for semaphores */
#if ((CY_CPU_CORTEX_M0P) || !(__CM0P_PRESENT))
    static uint32_t ipcSemaArray[CY_IPC_SEMA_COUNT / CY_IPC_SEMA_PER_WORD];
    (void) Cy_IPC_Sema_Init(CY_IPC_CHAN_SEMA, CY_IPC_SEMA_COUNT, ipcSemaArray);
#else
    (void) Cy_IPC_Sema_Init(CY_IPC_CHAN_SEMA, 0ul, NULL);
#endif
}

/*******************************************************************************
* Function Name: Cy_IPC_SystemPipeInit
****************************************************************************//**
*
* Initializes the system pipes. The system pipes are used by BLE and Flash.
* \note The function should be called on all CPUs.
*
* This function is called in the SystemInit() function. If the default startup
* file is not used, or SystemInit() is not called in your project,
* call the following three functions prior to executing any flash or EmEEPROM
* write or erase operation:
*  -# Cy_IPC_SystemSemaInit()
*  -# Cy_IPC_SystemPipeInit()
*  -# Cy_Flash_Init()
*
* Also this function is called to support BLE host/controller communication.
*
*******************************************************************************/
void Cy_IPC_SystemPipeInit(void)
{
     Cy_IPC_Pipe_Config(cy_ipc_pipe_sysEpArray);

#if (CY_IPC_CYPIPE_ENABLE)
    uint32_t intr;

    static cy_ipc_pipe_callback_ptr_t cy_ipc_pipe_sysCbArray[CY_IPC_CYPIPE_CLIENT_CNT];

    static const cy_stc_ipc_pipe_config_t systemPipeConfig = CY_CYPIPE_DEFAULT_CONFIG;

    intr = Cy_SysLib_EnterCriticalSection();

    Cy_IPC_Pipe_Init(&systemPipeConfig);

    Cy_SysLib_ExitCriticalSection(intr);
#endif /* CY_IPC_CYPIPE_ENABLE */
}

#if (CY_IPC_CYPIPE_ENABLE)
/*******************************************************************************
* Function Name: Cy_IPC_SystemPipeIsr
****************************************************************************//**
*
* This is the interrupt service routine for the system pipe.
*
*******************************************************************************/
static void Cy_IPC_SystemPipeIsr(void)
{
    Cy_IPC_Pipe_ExecuteCallback(CY_IPC_EP_CYPIPE_ADDR);
}
#endif /* CY_IPC_CYPIPE_ENABLE */
/*******************************************************************************
* Function Name: IPC_UserPipeInit
****************************************************************************//**
*
* Initializes the user pipes.
* \note The function should be called on all CPUs.
*
*******************************************************************************/
void IPC_UserPipeInit(void)
{
    uint32_t intr;

    intr = Cy_SysLib_EnterCriticalSection();

    static cy_ipc_pipe_callback_ptr_t ipc_pipe_CbArray[CY_IPC_USRPIPE_CLIENT_CNT];

    static const cy_stc_ipc_pipe_config_t userPipeConfig = USRPIPE_CONFIG;

    Cy_IPC_Pipe_Config(cy_ipc_pipe_sysEpArray);

    Cy_IPC_Pipe_Init(&userPipeConfig);
    
    Cy_SysLib_ExitCriticalSection(intr);
}
/*******************************************************************************
* Function Name: IPC_UserPipeIsr
****************************************************************************//**
*
* This is the interrupt service routine for the system pipe.
*
*******************************************************************************/
void IPC_UserPipeIsr(void)
{
    Cy_IPC_Pipe_ExecCallback(&cy_ipc_pipe_sysEpArray[IPC_EP_USRPIPE_ADDR]);
}
/* [] END OF FILE */

